'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.createMapScopeToProps = createMapScopeToProps;
exports.wrapWithDecorators = wrapWithDecorators;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createMapScopeToProps(bindings) {
    return function mapScopeToProps(scope) {
        return Object.keys(bindings).reduce(function (props, propName) {
            // We expect that properties which are bound with & are functions
            if (bindings[propName] === '&') {
                props[propName] = function (event) {
                    return scope[propName]({ $event: event });
                };
            } else {
                props[propName] = scope[propName];
            }

            return props;
        }, {});
    };
}

function wrapWithDecorators(decorators) {
    var compose = function compose(render, decorate) {
        // We create a new render function which takes props from outside
        var renderDecorator = function renderDecorator(props) {
            // Function which takes the passed props to render the component
            var renderComponentWithProps = function renderComponentWithProps() {
                return render(props);
            };

            // Pass the render function for component rendering to the decorate function
            // as render function
            return decorate(renderComponentWithProps);
        };

        return renderDecorator;
    };

    return function withDecorators(Component) {
        var render = function render(props) {
            return _react2.default.createElement(Component, props);
        };

        return decorators.reduce(compose, render);
    };
}